/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  The module contents functions for power fail processing.

  -------------------------------------------------------------------------

                $RCSfile: PwrfDrv.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.6 $  $Date: 2005/01/26 12:20:34 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/02/12 -as:   Start der Implementierung

****************************************************************************/

#include <XC161.h>
#include <intrins.h>

#include "generic/global.h"
#include "../include/PwrfDrv.h"

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

sbit    PowerFailPin = P2^15;

// Power Fail is signaled with rising edge
#define PowerFailState (PowerFailPin)

// interrupt register 
#define POWER_FAIL_INT_NUM         0x1F        // IntNum CC15IO
#define POWER_FAIL_INT_REG_MD      CC1_M3      // Mode Register
#define POWER_FAIL_INT_MASK        0xF000      // KanalMaske CC15IO
#define POWER_FAIL_INT_MODE        0x1000      // IntMode CC15IO (Capture RisingEdge)
#define POWER_FAIL_INT_REG_IC      CC1_CC15IC  // Interrupt Control Register
#define POWER_FAIL_INT_BIT_IC_IE   CC1_CC15IC_IE
#define POWER_FAIL_INT_BIT_IC_IR   CC1_CC15IC_IR
#define POWER_FAIL_INT_INTLVL      11          // interrupt level
#define POWER_FAIL_INT_GRPXLVL     0           // ext. group prio (XC16x spec)
#define POWER_FAIL_INT_GRPLVL      3           // group prio

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

static tfpPwrFailCbHandler fpPowerFailHandler_l;
static bit bitPowerFailFlag_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <Power Fail Process>                                */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Die Klasse enthlt Funktionen zum Schtzen eines kritischen 
// Abschnitts vor Unterbrechnung durch Power Fail.
//
/***************************************************************************/

//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

// -------------------------------------------------------------------------
//
// Function:     PowerFailInit
//
// Description:  The function initializes the power fail process. 
//               After called this function if supply voltage falls under
//               power fail level the CPU starts the process function.
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (19.04.2004 A.S.)
//
// -------------------------------------------------------------------------

void  PUBLIC PowerFailInit (void)
{

    fpPowerFailHandler_l = NULL;
    bitPowerFailFlag_l   = 0;

    // Power Fail Interrupt init.
    POWER_FAIL_INT_REG_IC  = 0;
    POWER_FAIL_INT_REG_MD &= ~POWER_FAIL_INT_MASK;       // clear MOD bits, keep all other
    POWER_FAIL_INT_REG_MD |= POWER_FAIL_INT_MODE;        // set MOD, capture on rising edge
    POWER_FAIL_INT_REG_IC  = (POWER_FAIL_INT_GRPXLVL<<8) | (POWER_FAIL_INT_INTLVL<<2) | POWER_FAIL_INT_GRPLVL | 0x40;

    // The process function will only be signalled at a rising edge
    // If the voltage is alraedy below the power-fail-level, it will not 
    // trigger an interrupt 
    // Therefore check manual and start the process  .
    if (PowerFailState)
    {
       POWER_FAIL_INT_BIT_IC_IR = 1;
    }
}


// -------------------------------------------------------------------------
//
// Function:     PowerFailShutDown
//
// Description:  The function finishs the power fail process. 
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (19.04.2004 A.S.)
//
// -------------------------------------------------------------------------

void  PUBLIC PowerFailShutDown (void)
{
    // Task will not call again. 
    POWER_FAIL_INT_REG_IC  = 0;
}

// -------------------------------------------------------------------------
//
// Function:     PowerFailGetPowerFailState
//
// Description:  The function returns the power fail state.
//
//
// Parameters:   none
//
// 
// Returns:      TRUE  --> Power Fail (Voltage is not in save region)
//               FALSE --> Power Good
//
//
// State:        
//
// -------------------------------------------------------------------------

BYTE PUBLIC PowerFailGetPowerFailState(void)
{
    return ( (PowerFailState)?TRUE:FALSE );
}

// -------------------------------------------------------------------------
//
// Function:     PowerFailInstallHandler
//
// Description:  The function installs an power fail handler of application.
//
//
// Parameters:   pointer to power fail handler of application
//
// 
// Returns:      none
//
//
// State:        tested (19.04.2004 A.S.)
//
// -------------------------------------------------------------------------

void PUBLIC PowerFailInstallHandler(tfpPwrFailCbHandler fpPowerFailHandler_p)
{
    POWER_FAIL_INT_BIT_IC_IE = 0;

    fpPowerFailHandler_l = fpPowerFailHandler_p;

    POWER_FAIL_INT_BIT_IC_IE = 1;
}

// -------------------------------------------------------------------------
//
// Function:     PowerFailStartCriticalSection
//
// Description:  The function protects a section against interruption
//               by power fail.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (19.04.2004 A.S.)
//
// -------------------------------------------------------------------------

void  PUBLIC PowerFailStartCriticalSection (void)
{
      bitPowerFailFlag_l = 1; 
}

// -------------------------------------------------------------------------
//
// Function:     PowerFailEndCriticalSection
//
// Description:  The function leaves a critical section.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (19.04.2004 A.S.)
//
// -------------------------------------------------------------------------

void  PUBLIC PowerFailEndCriticalSection (void)
{
      bitPowerFailFlag_l = 0; 
      // request power fail handler if power fail
      // was signaled during critical section
      POWER_FAIL_INT_BIT_IC_IE = 1; 
}

// -------------------------------------------------------------------------
//
// Function:     PowerFailEnableObservance
//
// Description:  The function enables the power fail event service.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        
//
// -------------------------------------------------------------------------

void PUBLIC PowerFailEnableObservance(void)
{
   POWER_FAIL_INT_BIT_IC_IR = 0;
   _nop_();
   POWER_FAIL_INT_BIT_IC_IE = 1;
}

// -------------------------------------------------------------------------
//
// Function:     PowerFailDisableObservance
//
// Description:  The function disables the power fail event service.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        
//
// -------------------------------------------------------------------------

void PUBLIC PowerFailDisableObservance(void)
{
   POWER_FAIL_INT_BIT_IC_IE = 0;
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

// -------------------------------------------------------------------------
//
// Function:     PowerFailProcess
//
// Description:  The function is called by interrupt if supply voltage is
//               falled under power fail level. 
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (19.04.2004 A.S.)
//
// -------------------------------------------------------------------------

static void ISR_PowerFailProcess (void) interrupt POWER_FAIL_INT_NUM
{
   // To prevent recalling power fail handler disable
   // power fail interrupt.
   POWER_FAIL_INT_BIT_IC_IE = 0;

   // application- specific reaction
   if (fpPowerFailHandler_l)
   {
      // If power fail during critical section then pass
      // 1 to user callback. In all over cases pass 0 to
      // user callback.
      fpPowerFailHandler_l(bitPowerFailFlag_l);
   }

   // Power Fail during critical section?
   if (bitPowerFailFlag_l)
   {
      // request power fail again
      POWER_FAIL_INT_BIT_IC_IR = 1;
   }
}


